@@ -52,6 +52,8 @@ class AbstractConnection(Generic[Node]):
     CONNECTION_TYPE: ClassVar[ConnectionType] = ConnectionType.NONE
     node: Node
     message_tracker: Optional[MessageTracker]
+    format_connection_desc: str
+    connection_repr: str
 
     def __init__(self, socket_connection: AbstractSocketConnectionProtocol, node: Node) -> None:
         self.socket_connection = socket_connection
@@ -107,10 +109,13 @@ class AbstractConnection(Generic[Node]):
         self._debug_message_tracker = defaultdict(int)
         self._last_debug_message_log_time = time.time()
 
+        self.processing_message_index = 0
+
         self.peer_model: Optional[OutboundPeerModel] = None
 
         self._is_authenticated = False
         self.account_id: Optional[str] = None
+        self.tier_name: Optional[str] = None
 
         self._close_waiter: Optional[Future] = None
         self.format_connection()
@@ -286,6 +291,8 @@ class AbstractConnection(Generic[Node]):
         messages_processed = defaultdict(int)
         total_bytes_processed = 0
 
+        self.processing_message_index = 0
+
         while True:
             input_buffer_len_before = self.inputbuf.length
             is_full_msg = False
@@ -437,6 +444,8 @@ class AbstractConnection(Generic[Node]):
             else:
                 self.num_bad_messages = 0
 
+            self.processing_message_index += 1
+
         performance_utils.log_operation_duration(msg_handling_logger,
                                                  "Message handlers",
                                                  start_time,
@@ -638,13 +647,14 @@ class AbstractConnection(Generic[Node]):
         if self.is_alive():
             raise ConnectionStateError("Connection is still alive after closed", self)
 
-    def set_account_id(self, account_id: Optional[str]):
+    def set_account_id(self, account_id: Optional[str], tier_name: str):
         if self._is_authenticated and account_id != self.account_id:
             raise ConnectionAuthenticationError(
                 f"Invalid account id {account_id} is different than connection account id: {self.account_id}")
 
         if not self._is_authenticated:
             self.account_id = account_id
+        self.tier_name = tier_name
 
     def get_backlog_size(self) -> int:
         output_buffer_backlog = self.outputbuf.length
@@ -653,20 +663,10 @@ class AbstractConnection(Generic[Node]):
         return output_buffer_backlog + socket_buffer_backlog
 
     def format_connection(self) -> None:
-        if self.CONNECTION_TYPE in ConnectionType.RELAY_ALL:
-            _short_connection_type = "R"
-
-        if self.CONNECTION_TYPE in ConnectionType.BLOCKCHAIN_NODE:
-            _short_connection_type = "B"
-
-        if self.CONNECTION_TYPE in ConnectionType.REMOTE_BLOCKCHAIN_NODE:
-            _short_connection_type = "RemoteB"
-
-        if self.CONNECTION_TYPE in ConnectionType.GATEWAY or self.network_num != constants.ALL_NETWORK_NUM:
-            _short_connection_type = "G"
-
-        _short_connection_type = self.CONNECTION_TYPE.name
-        self.format_connection_desc = "{} - {}".format(self.peer_desc, _short_connection_type)
+        self.format_connection_desc = "{} - {}".format(
+            self.peer_desc, self.CONNECTION_TYPE.format_short()
+        )
+        self.connection_repr = repr(self)
 
     def _pong_msg_timeout(self) -> None:
         if self.is_alive():
@@ -703,5 +703,6 @@ class AbstractConnection(Generic[Node]):
         return "<not available>"
 
     def _log_message(self, level: LogLevel, message, *args, **kwargs):
-        args = (HAS_PREFIX, f"[{self}]",) + args
-        logger.log(level, message, *args, **kwargs)
+        logger.log(
+            level, message, HAS_PREFIX, self.connection_repr, *args, **kwargs
+        )
