@@ -24,12 +24,13 @@ from bxcommon.models.outbound_peer_model import OutboundPeerModel
 from bxcommon.network.abstract_socket_connection_protocol import AbstractSocketConnectionProtocol
 from bxcommon.network.ip_endpoint import IpEndpoint
 from bxcommon.network.peer_info import ConnectionPeerInfo
-from bxcommon.network.socket_connection_state import SocketConnectionState
+from bxcommon.network.socket_connection_state import SocketConnectionStates
 from bxcommon.services import sdn_http_service
 from bxcommon.services.broadcast_service import BroadcastService, \
     BroadcastOptions
 from bxcommon.services.threaded_request_service import ThreadedRequestService
 from bxcommon.services.transaction_service import TransactionService
+from bxcommon.storage.serialized_message_cache import SerializedMessageCache
 from bxcommon.utils import memory_utils, convert, performance_utils
 from bxcommon.utils.alarm_queue import AlarmQueue, AlarmId
 from bxcommon.utils.blockchain_utils import bdn_tx_to_bx_tx
@@ -45,7 +46,7 @@ from bxutils import log_messages
 from bxutils import logging
 from bxutils.exceptions.connection_authentication_error import \
     ConnectionAuthenticationError
-from bxutils.logging import LogRecordType
+from bxutils.logging import LogRecordType, LogLevel
 from bxutils.services.node_ssl_service import NodeSSLService
 from bxutils.ssl.extensions import extensions_factory
 from bxutils.ssl.ssl_certificate_type import SSLCertificateType
@@ -93,6 +94,7 @@ class AbstractNode:
             self.connection_pool = ConnectionPool()
 
         self.should_force_exit = False
+        self.should_restart_on_high_memory = False
 
         self.num_retries_by_ip: Dict[Tuple[str, int], int] = defaultdict(int)
 
@@ -144,6 +146,8 @@ class AbstractNode:
         self._last_responsiveness_check_log_time = time.time()
         self._last_responsiveness_check_details = {}
         self.gc_logging_enabled = False
+        self.serialized_message_cache = SerializedMessageCache(self.alarm_queue)
+
         self.alarm_queue.register_alarm(constants.RESPONSIVENESS_CHECK_INTERVAL_S, self._responsiveness_check_log)
 
     def get_sdn_address(self):
@@ -401,7 +405,7 @@ class AbstractNode:
     def fire_alarms(self) -> float:
         time_to_next = self.alarm_queue.fire_ready_alarms()
         if time_to_next is not None:
-            return max(time_to_next, constants.MIN_SLEEP_TIMEOUT)
+            return time_to_next
         else:
             return constants.MAX_EVENT_LOOP_TIMEOUT
 
@@ -492,10 +496,6 @@ class AbstractNode:
         node_info_statistics.set_node(self)
         self.alarm_queue.register_alarm(constants.FIRST_STATS_INTERVAL_S, node_info_statistics.flush_info)
 
-    def init_memory_stats_logging(self):
-        memory_statistics.set_node(self)
-        memory_statistics.start_recording(self.record_mem_stats)
-
     def cleanup_memory_stats_logging(self):
         memory_statistics.stop_recording()
 
@@ -511,13 +511,22 @@ class AbstractNode:
                 conn.socket_connection.send()
         return self.FLUSH_SEND_BUFFERS_INTERVAL
 
-    def record_mem_stats(self):
+    def record_mem_stats(self, low_threshold: int, medium_threshold: int, high_threshold: int):
         """
         When overridden, records identified memory stats and flushes them to std out
         :returns memory stats flush interval
         """
-        self.connection_pool.log_connection_pool_mem_stats()
-        return memory_statistics.flush_info()
+        total_memory = memory_utils.get_app_memory_usage()
+        if total_memory > low_threshold:
+            gc.collect()
+            total_memory = memory_utils.get_app_memory_usage()
+        self._record_mem_stats(total_memory > medium_threshold)
+
+        return memory_statistics.flush_info(high_threshold)
+
+    def _record_mem_stats(self, include_data_structure_memory: bool = False):
+        if include_data_structure_memory:
+            self.connection_pool.log_connection_pool_mem_stats()
 
     def set_node_config_opts_from_sdn(self, opts: CommonOpts) -> None:
         blockchain_networks: Dict[int, BlockchainNetworkModel] = opts.blockchain_networks
@@ -529,17 +538,15 @@ class AbstractNode:
             )
         bdn_tx_to_bx_tx.init(blockchain_networks)
 
-    def dump_memory_usage(self):
-        total_mem_usage = memory_utils.get_app_memory_usage()
-        if total_mem_usage >= self.next_report_mem_usage_bytes:
+    def dump_memory_usage(self, total_memory: int, threshold: int):
+        if total_memory > threshold and logger.isEnabledFor(LogLevel.DEBUG):
             node_size = self.get_node_memory_size()
             memory_logger.debug(
                 "Application consumed {} bytes which is over set limit {} bytes. Detailed memory report: {}",
-                total_mem_usage,
-                self.next_report_mem_usage_bytes,
+                total_memory,
+                threshold,
                 node_size
             )
-            self.next_report_mem_usage_bytes = total_mem_usage + constants.MEMORY_USAGE_INCREASE_FOR_NEXT_REPORT_BYTES
 
     def get_node_memory_size(self):
         return memory_utils.get_detailed_object_size(self)
@@ -580,6 +587,18 @@ class AbstractNode:
         logger.trace("Fetching SSL certificate for: {} ({}).", endpoint, connection_type)
         return self.node_ssl_service.create_ssl_context(SSLCertificateType.PRIVATE)
 
+    @abstractmethod
+    def reevaluate_transaction_streamer_connection(self) -> None:
+        raise NotImplementedError
+
+    @abstractmethod
+    def on_new_subscriber_request(self) -> None:
+        raise NotImplementedError
+
+    @abstractmethod
+    def init_memory_stats_logging(self):
+        raise NotImplementedError
+
     @abstractmethod
     def sync_tx_services(self):
         self.start_sync_time = time.time()
@@ -618,6 +637,9 @@ class AbstractNode:
         node_privileges = extensions_factory.get_node_privileges(cert)
         return AuthenticatedPeerInfo(connection_type, peer_id, account_id, node_privileges)
 
+    def _should_log_closed_connection(self, _connection: AbstractConnection) -> bool:
+        return True
+
     def _destroy_conn(self, conn: AbstractConnection):
         """
         Clean up the associated connection and update all data structures tracking it.
@@ -631,9 +653,11 @@ class AbstractNode:
 
         :param conn connection to destroy
         """
-        self.log_closed_connection(conn)
 
-        should_retry = SocketConnectionState.DO_NOT_RETRY not in conn.socket_connection.state
+        if self._should_log_closed_connection(conn):
+            self.log_closed_connection(conn)
+
+        should_retry = SocketConnectionStates.DO_NOT_RETRY not in conn.socket_connection.state
 
         logger.debug("Breaking connection to {}. Attempting retry: {}", conn, should_retry)
         conn.dispose()
