@@ -18,7 +18,20 @@ class ConnectionType(SerializableFlag):
     GATEWAY = INTERNAL_GATEWAY | EXTERNAL_GATEWAY
 
     def __str__(self):
-        return self.name
+        return str(self.name)
 
     def __format__(self, format_spec):
+        return str(self.name)
+
+    def format_short(self):
+        cls = self.__class__
+        if self in cls.RELAY_ALL:
+            return "R"
+        if self in cls.BLOCKCHAIN_NODE:
+            return "B"
+        if self in cls.REMOTE_BLOCKCHAIN_NODE:
+            return "RemoteB"
+        if self in cls.GATEWAY:
+            return "G"
+
         return self.name
