@@ -2,9 +2,6 @@ import os
 import platform
 import socket
 
-from bxcommon.utils import crypto
-from bxcommon.utils.object_hash import Sha256Hash
-
 PLATFORM_LINUX = "linux"
 PLATFORM_MAC = "darwin"
 DEFAULT_TEXT_ENCODING = "utf-8"
@@ -37,13 +34,17 @@ OUTPUT_BUFFER_BATCH_MAX_HOLD_TIME = 0.05
 
 FULL_QUOTA_PERCENTAGE = 100
 
+WS_PROVIDER_MAX_QUEUE_SIZE = 1000
+WS_MIN_RECONNECT_TIMEOUT_S = 1
+WS_RECONNECT_TIMEOUTS = [1, 2, 3, 5, 8, 13]
+WS_MAX_CONNECTION_TIMEOUT_S = 2
+
 # The unsigned integer transaction SID representing null.
 # If changing, also change in bxapi/constants.py
 NULL_TX_SID = 0
 NULL_TX_SIDS = {NULL_TX_SID}
 NULL_TX_TIMESTAMP = 0
 TX_SID_INTERVAL = 10000000
-UNKNOWN_TRANSACTION_HASH: Sha256Hash = Sha256Hash(bytearray(b"\xff" * crypto.SHA256_HASH_LEN))
 # </editor-fold>
 
 # <editor-fold desc="Connection Management">
@@ -63,6 +64,7 @@ MAX_CONNECT_TIMEOUT_INCREASE = 7
 RECV_BUFSIZE = 1024 * 1024
 MAX_BAD_MESSAGES = 3
 PING_INTERVAL_S = 60
+PING_PONG_TRESHOLD = 0.5
 # </editor-fold>
 
 # <editor-fold desc="Logging">
@@ -96,6 +98,7 @@ MSG_TYPE_LEN = 12
 
 QUOTA_FLAG_LEN = UL_TINY_SIZE_IN_BYTES
 SID_LEN = UL_INT_SIZE_IN_BYTES
+TRANSACTION_FLAG_LEN = UL_SHORT_SIZE_IN_BYTES
 
 SID_EXPIRE_TIME_SECONDS = 3 * 24 * 60 * 60
 
@@ -223,6 +226,9 @@ NETWORK_OPERATION_DURATION_WARN_THRESHOLD_S = 0.5
 GC_DURATION_WARN_THRESHOLD = 0.1
 TX_SERVICE_SET_TXS_WARN_THRESHOLD = 0.1
 
+SERIALIZED_MESSAGE_CACHE_EXPIRE_TIME_S = 10
+USE_SERIALIZED_MESSAGE_CACHE = False
+
 # </editor-fold>
 
 # <editor-fold desc="Default Values">
@@ -246,6 +252,7 @@ TX_SERVICE_CHECK_NETWORKS_SYNCED_S = 10 * 60
 LAST_MSG_FROM_RELAY_THRESHOLD_S = 30
 PING_TIMEOUT_S = 2
 
+GATEWAY_SWAP_RELAYS_LATENCY_THRESHOLD_MS = 3
 NODE_LATENCY_THRESHOLD_MS = 2
 FASTEST_PING_LATENCY_THRESHOLD_PERCENT = 0.2
 UPDATE_TX_SERVICE_FULLY_SYNCED_S = 1
@@ -293,6 +300,7 @@ UNASSIGNED_NETWORK_NUMBER = -1
 PING_PONG_REPLY_TIMEOUT_S = 5 * 60
 
 NODE_COUNTRY_ATTRIBUTE_NAME = "country"
+NODE_REGION_ATTRIBUTE_NAME = "region"
 NODE_COUNTRY_CHINA = "China"
 
 NODE_SHUTDOWN_TIMEOUT_S = 30
@@ -312,5 +320,17 @@ MEM_STATS_OBJECT_COUNT_THRESHOLD = 2000
 
 TX_FROM_PUB_API_TO_RELAY_THRESHOLD_S = 2.0
 TRANSACTION_STREAMER_ATTRIBUTE_NAME = "tx_streamer"
+PRIVATE_IP_ATTRIBUTE_NAME = "using_private_ip"
 
 EPOCH_DATE: str = "1970-01-01"
+END_DATE: str = "2999-01-01"
+
+RPC_SUBSCRIBER_MAX_QUEUE_SIZE = 1000
+
+WS_DEFAULT_PORT = 28332
+
+GC_LOW_MEMORY_THRESHOLD = 1024 * 1024 * 1024 * 3
+GC_MEDIUM_MEMORY_THRESHOLD = 1024 * 1024 * 1024 * 3.5
+GC_HIGH_MEMORY_THRESHOLD = 1024 * 1024 * 1024 * 4
+
+BLOCKS_FAILED_VALIDATION_HISTORY_SIZE = 10
