@@ -18,6 +18,20 @@ from bxcommon.messages.bloxroute.v12.bloxroute_message_factory_v12 import bloxro
 from bxcommon.messages.bloxroute.v12.message_converter_factory_v12 import message_converter_factory_v12
 from bxcommon.messages.bloxroute.v13.bloxroute_message_factory_v13 import bloxroute_message_factory_v13
 from bxcommon.messages.bloxroute.v13.message_converter_factory_v13 import message_converter_factory_v13
+from bxcommon.messages.bloxroute.v14.bloxroute_message_factory_v14 import bloxroute_message_factory_v14
+from bxcommon.messages.bloxroute.v14.message_converter_factory_v14 import message_converter_factory_v14
+from bxcommon.messages.bloxroute.v15.bloxroute_message_factory_v15 import bloxroute_message_factory_v15
+from bxcommon.messages.bloxroute.v15.message_converter_factory_v15 import message_converter_factory_v15
+from bxcommon.messages.bloxroute.v16.bloxroute_message_factory_v16 import bloxroute_message_factory_v16
+from bxcommon.messages.bloxroute.v16.message_converter_factory_v16 import message_converter_factory_v16
+from bxcommon.messages.bloxroute.v17.bloxroute_message_factory_v17 import bloxroute_message_factory_v17
+from bxcommon.messages.bloxroute.v17.message_converter_factory_v17 import message_converter_factory_v17
+from bxcommon.messages.bloxroute.v18.message_converter_factory_v18 import message_converter_factory_v18
+from bxcommon.messages.bloxroute.v18.bloxroute_message_factory_v18 import bloxroute_message_factory_v18
+from bxcommon.messages.bloxroute.v19.bloxroute_message_factory_v19 import bloxroute_message_factory_v19
+from bxcommon.messages.bloxroute.v19.message_converter_factory_v19 import message_converter_factory_v19
+from bxcommon.messages.bloxroute.v20.message_converter_factory_v20 import message_converter_factory_v20
+from bxcommon.messages.bloxroute.v20.bloxroute_message_factory_v20 import bloxroute_message_factory_v20
 from bxcommon.messages.versioning.abstract_version_manager import AbstractVersionManager
 
 
@@ -34,8 +48,13 @@ class _BloxrouteVersionManager(AbstractVersionManager):
         11: message_converter_factory_v11,
         12: message_converter_factory_v12,
         13: message_converter_factory_v13,
-
-
+        14: message_converter_factory_v14,
+        15: message_converter_factory_v15,
+        16: message_converter_factory_v16,
+        17: message_converter_factory_v17,
+        18: message_converter_factory_v18,
+        19: message_converter_factory_v19,
+        20: message_converter_factory_v20
     }
     _PROTOCOL_TO_FACTORY_MAPPING = {
         6: bloxroute_message_factory_v6,
@@ -46,7 +65,14 @@ class _BloxrouteVersionManager(AbstractVersionManager):
         11: bloxroute_message_factory_v11,
         12: bloxroute_message_factory_v12,
         13: bloxroute_message_factory_v13,
-        14: bloxroute_message_factory
+        14: bloxroute_message_factory_v14,
+        15: bloxroute_message_factory_v15,
+        16: bloxroute_message_factory_v16,
+        17: bloxroute_message_factory_v17,
+        18: bloxroute_message_factory_v18,
+        19: bloxroute_message_factory_v19,
+        20: bloxroute_message_factory_v20,
+        21: bloxroute_message_factory
     }
 
     def __init__(self) -> None:
