@@ -1,8 +1,9 @@
+from typing import Optional
+
 from bxcommon import constants
 from bxcommon.messages.bloxroute.abstract_bloxroute_message import AbstractBloxrouteMessage
 from bxcommon.messages.bloxroute.protocol_version import PROTOCOL_VERSION
 from bxcommon.utils.message_buffer_builder import PayloadElement, PayloadBlock
-from bxutils.logging import LogLevel
 
 
 class KeepAliveMessage(AbstractBloxrouteMessage):
@@ -17,7 +18,13 @@ class KeepAliveMessage(AbstractBloxrouteMessage):
     )
     KEEP_ALIVE_MESSAGE_LENGTH = KEEP_ALIVE_MESSAGE_BLOCK.size + constants.CONTROL_FLAGS_LEN
 
-    def __init__(self, msg_type, nonce=None, buf=None, payload_length=None) -> None:
+    def __init__(
+        self,
+        msg_type: bytes,
+        nonce: Optional[int] = None,
+        buf: Optional[bytearray] = None,
+        payload_length: Optional[int] = None
+    ) -> None:
         if payload_length is None:
             payload_length = self.KEEP_ALIVE_MESSAGE_LENGTH
         if buf is None:
@@ -26,19 +33,18 @@ class KeepAliveMessage(AbstractBloxrouteMessage):
         buf = self.KEEP_ALIVE_MESSAGE_BLOCK.build(buf, nonce=nonce)
 
         self.buf = buf
-        self._nonce = None
-        self._network_num = None
+        self._nonce: Optional[int] = None
+        self._network_num: Optional[int] = None
         self._memoryview = memoryview(buf)
         super(KeepAliveMessage, self).__init__(msg_type, payload_length, buf)
 
-    def __unpack(self):
+    def __unpack(self) -> None:
         contents = self.KEEP_ALIVE_MESSAGE_BLOCK.read(self._memoryview)
         self._nonce = contents.get("nonce")
 
-    def nonce(self):
+    def nonce(self) -> int:
         if self._nonce is None:
             self.__unpack()
-        return self._nonce
-
-    def log_level(self) -> LogLevel:
-        return LogLevel.DEBUG
+        nonce = self._nonce
+        assert nonce is not None
+        return nonce
