@@ -9,6 +9,7 @@ from bxcommon.messages.bloxroute.bloxroute_message_type import (
     BloxrouteMessageType,
 )
 from bxcommon.models.quota_type_model import QuotaType
+from bxcommon.models.transaction_flag import TransactionFlag
 from bxcommon.utils.object_hash import Sha256Hash
 
 
@@ -16,8 +17,8 @@ class TxMessage(AbstractBroadcastMessage):
     PAYLOAD_LENGTH = (
         AbstractBroadcastMessage.PAYLOAD_LENGTH
         + constants.SID_LEN
-        + constants.QUOTA_FLAG_LEN
-        + constants.UL_INT_SIZE_IN_BYTES
+        + constants.TRANSACTION_FLAG_LEN
+        + constants.DOUBLE_SIZE_IN_BYTES
     )
     MESSAGE_TYPE = BloxrouteMessageType.TRANSACTION
     EMPTY_TX_VAL = memoryview(bytes())
@@ -29,13 +30,13 @@ class TxMessage(AbstractBroadcastMessage):
         source_id: str = "",
         short_id: int = constants.NULL_TX_SID,
         tx_val: Union[bytearray, bytes, memoryview, None] = None,
-        quota_type: Optional[QuotaType] = None,
+        transaction_flag: Optional[TransactionFlag] = None,
         timestamp: Union[int, float] = constants.NULL_TX_TIMESTAMP,
         buf: Optional[Union[bytearray, memoryview]] = None,
     ):
         self._short_id = None
         self._tx_val: Optional[memoryview] = None
-        self._tx_quota_type = None
+        self._transaction_flag = None
         self._timestamp = None
 
         # override payload length for variable length message
@@ -44,12 +45,10 @@ class TxMessage(AbstractBroadcastMessage):
             self.PAYLOAD_LENGTH = (
                 AbstractBroadcastMessage.PAYLOAD_LENGTH
                 + constants.SID_LEN
-                + constants.QUOTA_FLAG_LEN
-                + constants.UL_INT_SIZE_IN_BYTES
+                + constants.TRANSACTION_FLAG_LEN
+                + constants.DOUBLE_SIZE_IN_BYTES
                 + len(tx_val)
             )
-        # pyre-fixme[6]: Expected `Optional[bytearray]` for 4th param but got
-        #  `Optional[Union[bytearray, memoryview]]`.
         super().__init__(message_hash, network_num, source_id, buf)
 
         if buf is None:
@@ -63,13 +62,13 @@ class TxMessage(AbstractBroadcastMessage):
             struct.pack_into("<L", self.buf, off, short_id)
             off += constants.SID_LEN
 
-            if quota_type is None:
-                quota_type = QuotaType.FREE_DAILY_QUOTA
-            struct.pack_into("<B", self.buf, off, quota_type.value)
-            off += constants.QUOTA_FLAG_LEN
+            if transaction_flag is None:
+                transaction_flag = TransactionFlag.NO_FLAGS
+            struct.pack_into("<H", self.buf, off, transaction_flag.value)
+            off += constants.TRANSACTION_FLAG_LEN
 
-            struct.pack_into("<L", self.buf, off, int(timestamp))
-            off += constants.UL_INT_SIZE_IN_BYTES
+            struct.pack_into("<d", self.buf, off, timestamp)
+            off += constants.DOUBLE_SIZE_IN_BYTES
 
             if tx_val is not None:
                 self.buf[off:off + len(tx_val)] = tx_val
@@ -82,7 +81,7 @@ class TxMessage(AbstractBroadcastMessage):
             f"network_num: {self.network_num()}, "
             f"compact: {self.is_compact()}, "
             f"source_id: {self.source_id_as_str()}, "
-            f"quota_type: {self.quota_type()}, "
+            f"transaction_flag: {self.transaction_flag()}, "
             f"timestamp: {self.timestamp()}"
             f">"
         )
@@ -104,8 +103,8 @@ class TxMessage(AbstractBroadcastMessage):
     def has_short_id(self) -> bool:
         return self.short_id() != constants.NULL_TX_SID
 
-    def quota_type(self) -> QuotaType:
-        if self._tx_quota_type is None:
+    def transaction_flag(self) -> TransactionFlag:
+        if self._transaction_flag is None:
             off = (
                 self.HEADER_LENGTH
                 + AbstractBroadcastMessage.PAYLOAD_LENGTH
@@ -113,23 +112,32 @@ class TxMessage(AbstractBroadcastMessage):
                 - constants.CONTROL_FLAGS_LEN
             )
 
-            (tx_quota_type_flag,) = struct.unpack_from("<B", self.buf, off)
-            self._tx_quota_type = QuotaType(tx_quota_type_flag)
-        assert self._tx_quota_type is not None
-        # pyre-fixme[7]: Expected `QuotaType` but got `None`.
-        return self._tx_quota_type
+            (transaction_flag, ) = struct.unpack_from("<H", self.buf, off)
+            self._transaction_flag = TransactionFlag(transaction_flag)
+        assert self._transaction_flag is not None
+        return TransactionFlag(self._transaction_flag)
 
-    def timestamp(self) -> int:
+    def set_transaction_flag(self, flag: TransactionFlag) -> None:
+        off = (
+            self.HEADER_LENGTH
+            + AbstractBroadcastMessage.PAYLOAD_LENGTH
+            + constants.SID_LEN
+            - constants.CONTROL_FLAGS_LEN
+        )
+        self._transaction_flag = flag
+        struct.pack_into("<H", self.buf, off, flag.value)
+
+    def timestamp(self) -> float:
         if self._timestamp is None:
             off = (
                 self.HEADER_LENGTH
                 + AbstractBroadcastMessage.PAYLOAD_LENGTH
                 + constants.SID_LEN
-                + constants.QUOTA_FLAG_LEN
+                + constants.TRANSACTION_FLAG_LEN
                 - constants.CONTROL_FLAGS_LEN
             )
-            (self._timestamp,) = struct.unpack_from("<L", self.buf, off)
-        # pyre-fixme[7]: Expected `int` but got `None`.
+            (self._timestamp,) = struct.unpack_from("<d", self.buf, off)
+        # pyre-fixme[7]: Expected `float` but got `None`.
         return self._timestamp
 
     def tx_val(self) -> memoryview:
@@ -141,8 +149,8 @@ class TxMessage(AbstractBroadcastMessage):
                     self.HEADER_LENGTH
                     + AbstractBroadcastMessage.PAYLOAD_LENGTH
                     + constants.SID_LEN
-                    + constants.QUOTA_FLAG_LEN
-                    + constants.UL_INT_SIZE_IN_BYTES
+                    + constants.TRANSACTION_FLAG_LEN
+                    + constants.DOUBLE_SIZE_IN_BYTES
                     - constants.CONTROL_FLAGS_LEN
                 )
                 self._tx_val = self._memoryview[
@@ -151,24 +159,23 @@ class TxMessage(AbstractBroadcastMessage):
                     - constants.CONTROL_FLAGS_LEN
                 ]
 
-        assert self._tx_val is not None
-        # pyre-fixme[7]: Expected `memoryview` but got `Optional[memoryview]`.
-        return self._tx_val
+        tx_val = self._tx_val
+        assert tx_val is not None
+        return tx_val
 
     def is_compact(self) -> bool:
         return self.tx_val() == self.EMPTY_TX_VAL
 
     def set_timestamp(self, timestamp: Union[float, int]):
-        timestamp = int(timestamp)
         self._timestamp = timestamp
         off = (
             self.HEADER_LENGTH
             + AbstractBroadcastMessage.PAYLOAD_LENGTH
             + constants.SID_LEN
-            + constants.QUOTA_FLAG_LEN
+            + constants.TRANSACTION_FLAG_LEN
             - constants.CONTROL_FLAGS_LEN
         )
-        struct.pack_into("<L", self.buf, off, timestamp)
+        struct.pack_into("<d", self.buf, off, timestamp)
 
     def clear_short_id(self):
         off = (
@@ -184,10 +191,10 @@ class TxMessage(AbstractBroadcastMessage):
             self.HEADER_LENGTH
             + AbstractBroadcastMessage.PAYLOAD_LENGTH
             + constants.SID_LEN
-            + constants.QUOTA_FLAG_LEN
+            + constants.TRANSACTION_FLAG_LEN
             - constants.CONTROL_FLAGS_LEN
         )
-        struct.pack_into("<L", self.buf, off, constants.NULL_TX_TIMESTAMP)
+        struct.pack_into("<d", self.buf, off, constants.NULL_TX_TIMESTAMP)
         self._timestamp = constants.NULL_TX_TIMESTAMP
 
     def clear_protected_fields(self):
@@ -197,3 +204,6 @@ class TxMessage(AbstractBroadcastMessage):
         """
         self.clear_short_id()
         self.clear_timestamp()
+
+    def quota_type(self) -> QuotaType:
+        return self.transaction_flag().get_quota_type()
