@@ -11,7 +11,14 @@ class AbstractMessageValidator:
     __metaclass__ = ABCMeta
 
     @abstractmethod
-    def validate(self, is_full_msg: bool,  msg_type: str, header_len: int, payload_len: int, input_buffer: InputBuffer) -> None:
+    def validate(
+        self,
+        is_full_msg: bool,
+        msg_type: str,
+        header_len: int,
+        payload_len: int,
+        input_buffer: InputBuffer
+    ) -> None:
         """
         Validates message payload length.
         Throws MessageValidationError is message is not valid
@@ -22,4 +29,3 @@ class AbstractMessageValidator:
         :param payload_len: message payload length
         :param input_buffer: input buffer
         """
-        pass
