@@ -1,13 +1,14 @@
 import asyncio
 import gc
 import sys
+import os
 
 from typing import Iterable, Optional, Type, Callable, List
 
 import uvloop
 
 from bxcommon.connections.abstract_node import AbstractNode
-from bxcommon.exceptions import TerminationError
+from bxcommon.exceptions import TerminationError, HighMemoryError
 from bxcommon.models.node_type import NodeType
 from bxcommon.network.node_event_loop import NodeEventLoop
 from bxcommon.services import sdn_http_service
@@ -77,6 +78,7 @@ def run_node(
         third_party_loggers=third_party_loggers,
         fluent_log_level=opts.log_level_fluentd,
         stdout_log_level=opts.log_level_stdout,
+        fluentd_tag_suffix=node_type.name.lower()
     )
     if node_init_tasks is None:
         node_init_tasks = common_init_tasks.init_tasks
@@ -89,10 +91,8 @@ def run_node(
     config.log_pid(process_id_file_path)
     gc.callbacks.append(gc_logger.gc_callback)
     # we disable GC generation cleanup.
-    # if memory goes out of control we need to find why or
-    # use gc.collect() every 30 minutes.
+    # use gc.collect() if memory exceeds threshold
     gc.disable()
-
     try:
         if opts.use_extensions:
             from bxcommon.utils.proxy import task_pool_proxy
@@ -112,8 +112,18 @@ def run_node(
         )
     except TerminationError:
         logger.fatal("Node terminated")
+    except HighMemoryError:
+        logger.info("Restarting node due to high memory")
+        _close_handles()
+        python = sys.executable
+        os.execl(python, python, *sys.argv)
     except Exception as e:  # pylint: disable=broad-except
         logger.fatal("Unhandled exception {} raised, terminating!", e)
+
+    _close_handles()
+
+
+def _close_handles() -> None:
     for handler in logger.handlers:
         if hasattr(handler, "close"):
             handler.close()
@@ -170,7 +180,8 @@ def _init_ssl_service(
     )
     node_ssl_service.blocking_load()
 
-    if node_ssl_service.has_valid_certificate(SSLCertificateType.PRIVATE):
+    if node_ssl_service.has_valid_certificate(SSLCertificateType.PRIVATE) \
+        and not node_ssl_service.should_renew_node_certificate():
         ssl_context = node_ssl_service.create_ssl_context(SSLCertificateType.PRIVATE)
     else:
         ssl_context = node_ssl_service.create_ssl_context(
