@@ -1,7 +1,8 @@
 import sys
 
 P2P_PROTOCOL_VERSION = 4
-ETH_PROTOCOL_VERSION = 63
+ETH_PROTOCOL_VERSION = 64
+SUPPORTED_ETH_PROTOCOL_VERSION = [ETH_PROTOCOL_VERSION, 63]
 CAPABILITIES = ((b"eth", ETH_PROTOCOL_VERSION),)
 BX_ETH_CLIENT_NAME = b"BloxRoute Gateway"
 
@@ -63,6 +64,7 @@ BLOCK_HASH_LEN = 32
 ADDRESS_LEN = 20
 MERKLE_ROOT_LEN = 32
 BLOOM_LEN = 256
+FIXED_LENGTH_FIELD_OFFSET = 2 * BLOCK_HASH_LEN + ADDRESS_LEN + 3 * MERKLE_ROOT_LEN + BLOOM_LEN + 9
 
 HANDSHAKE_TIMEOUT_SEC = 30
 PING_PONG_INTERVAL_SEC = 30
@@ -102,3 +104,6 @@ ETH_MINIMAL_SUB_TASK_TX_COUNT = 100
 # these sync messages are usually useless to the BDN
 ETH_SKIP_TRANSACTIONS_LEAD_TIME_S = 30
 ETH_SKIP_TRANSACTIONS_SIZE = 30000
+
+MAX_FUTURE_BLOCK_NUMBER: int = 100
+MAX_DIFFICULTY_CHANGE_PERCENT: int = 10
