@@ -28,6 +28,7 @@ class LogRecordType(Enum):
     TransactionAudit = "stats.transaction.audit"
     TransactionAuditSummary = "stats.transaction.audit.summary"
     TransactionStatus = "transaction.status"
+    TransactionTracing = "network_content.tx.tracing"
     BlockAudit = "stats.block.audit"
     BlockAuditSummary = "stats.block.audit.summary"
     AuditUpdates = "stats.audit.updates"
@@ -35,6 +36,8 @@ class LogRecordType(Enum):
     ExecutionTimerInfo = "stats.execution.timer.info"
     TaskDuration = "stats.task_duration"
     NetworkContent = "network_content.stats"
+    NetworkContentTx = "network_content.tx.stats"
+    NetworkContentBlock = "network_content.block.stats"
     RoutingService = "routing.service"
     ConnectionHealth = "stats.connection_health"
     PerformanceTroubleshooting = "stats.performance.responsiveness"
@@ -48,3 +51,8 @@ class LogRecordType(Enum):
     QuotaNotification = "stats.quota.notification"
     QuotaFillStatus = "stats.quota.fill"
     TransactionQuota = "transaction.quota"
+    TransactionFiltering = "transaction.filter"
+    TxBlockchainStatusInfo = "stats.transaction.status"
+    PrivateTransaction = "stats.cloudapi.private_transaction"
+    PaidTransaction = "stats.transaction.paid_transaction"
+    TxDetectionTimeLocation = "stats.transaction.tx_detection_time_location"
