@@ -6,6 +6,7 @@ from typing import TYPE_CHECKING, Set
 from bxcommon import constants
 from bxcommon.connections.connection_type import ConnectionType
 from bxcommon.connections.internal_node_connection import InternalNodeConnection
+from bxcommon.feed.feed import FeedKey
 from bxcommon.messages.bloxroute.abstract_cleanup_message import AbstractCleanupMessage
 from bxcommon.messages.bloxroute.bdn_performance_stats_message import BdnPerformanceStatsMessage
 from bxcommon.messages.bloxroute.blockchain_network_message import RefreshBlockchainNetworkMessage
@@ -19,6 +20,7 @@ from bxcommon.messages.bloxroute.txs_message import TxsMessage
 from bxcommon.messages.validation.message_size_validation_settings import MessageSizeValidationSettings
 from bxcommon.models.entity_type_model import EntityType
 from bxcommon.models.notification_code import NotificationCode
+from bxcommon.models.transaction_flag import TransactionFlag
 from bxcommon.network.abstract_socket_connection_protocol import AbstractSocketConnectionProtocol
 from bxcommon.services import sdn_http_service
 from bxcommon.utils import convert, performance_utils
@@ -30,7 +32,7 @@ from bxcommon.utils.stats.block_statistics_service import block_stats
 from bxcommon.utils.stats.transaction_stat_event_type import TransactionStatEventType
 from bxcommon.utils.stats.transaction_statistics_service import tx_stats
 from bxgateway import log_messages, gateway_constants
-from bxgateway.feed.new_transaction_feed import NewTransactionFeed, RawTransactionFeedEntry
+from bxcommon.feed.new_transaction_feed import NewTransactionFeed, RawTransactionFeedEntry
 from bxgateway.services.block_recovery_service import RecoveredTxsSource
 from bxgateway.services.gateway_transaction_service import MissingTransactions
 from bxgateway.utils.logging.status import status_log
@@ -54,6 +56,8 @@ class AbstractRelayConnection(InternalNodeConnection["AbstractGatewayNode"]):
 
     CONNECTION_TYPE = ConnectionType.RELAY_ALL
 
+    node: "AbstractGatewayNode"
+
     def __init__(self, sock: AbstractSocketConnectionProtocol, node: "AbstractGatewayNode"):
         super(AbstractRelayConnection, self).__init__(sock, node)
 
@@ -144,7 +148,7 @@ class AbstractRelayConnection(InternalNodeConnection["AbstractGatewayNode"]):
                 TransactionStatEventType.TX_RECEIVED_BY_GATEWAY_FROM_PEER_IGNORE_SEEN,
                 network_num,
                 short_id,
-                peer=stats_format.connection(self),
+                peers=[self],
                 is_compact_transaction=False
             )
             self.log_trace("Transaction has already been seen: {}", tx_hash)
@@ -157,7 +161,7 @@ class AbstractRelayConnection(InternalNodeConnection["AbstractGatewayNode"]):
                 TransactionStatEventType.TX_RECEIVED_BY_GATEWAY_FROM_PEER_IGNORE_SEEN,
                 network_num,
                 short_id,
-                peer=stats_format.connection(self),
+                peers=[self],
                 is_compact_transaction=is_compact
             )
             return
@@ -167,7 +171,7 @@ class AbstractRelayConnection(InternalNodeConnection["AbstractGatewayNode"]):
             TransactionStatEventType.TX_RECEIVED_BY_GATEWAY_FROM_PEER,
             network_num,
             short_id,
-            peer=stats_format.connection(self),
+            peers=[self],
             is_compact_transaction=msg.is_compact()
         )
         gateway_transaction_stats_service.log_transaction_from_relay(
@@ -177,7 +181,9 @@ class AbstractRelayConnection(InternalNodeConnection["AbstractGatewayNode"]):
         )
 
         if processing_result.assigned_short_id:
-            was_missing = self.node.block_recovery_service.check_missing_sid(short_id, RecoveredTxsSource.TXS_RECEIVED_FROM_BDN)
+            was_missing = self.node.block_recovery_service.check_missing_sid(
+                short_id, RecoveredTxsSource.TXS_RECEIVED_FROM_BDN
+            )
             attempt_recovery |= was_missing
             tx_stats.add_tx_by_hash_event(
                 tx_hash,
@@ -187,14 +193,6 @@ class AbstractRelayConnection(InternalNodeConnection["AbstractGatewayNode"]):
                 was_missing=was_missing
             )
             gateway_transaction_stats_service.log_short_id_assignment_processed()
-        elif not short_id:
-            tx_stats.add_tx_by_hash_event(
-                tx_hash,
-                TransactionStatEventType.TX_SHORT_ID_EMPTY_IN_MSG_FROM_RELAY,
-                network_num,
-                short_id,
-                peer=stats_format.connection(self)
-            )
 
         if not is_compact and processing_result.existing_contents:
             gateway_transaction_stats_service.log_redundant_transaction_content()
@@ -204,16 +202,19 @@ class AbstractRelayConnection(InternalNodeConnection["AbstractGatewayNode"]):
             gateway_bdn_performance_stats_service.log_tx_from_bdn(
                 not self.node.is_gas_price_above_min_network_fee(tx_contents)
             )
-            attempt_recovery |= self.node.block_recovery_service.check_missing_tx_hash(tx_hash, RecoveredTxsSource.TXS_RECEIVED_FROM_BDN)
-
-            self.publish_new_transaction(
-                tx_hash, tx_contents
+            attempt_recovery |= self.node.block_recovery_service.check_missing_tx_hash(
+                tx_hash, RecoveredTxsSource.TXS_RECEIVED_FROM_BDN
             )
 
-            if self.node.node_conn is not None:
+            self.publish_new_transaction(
+                tx_hash, tx_contents, TransactionFlag.LOCAL_REGION in msg.transaction_flag()
+            )
+
+            if self.node.has_active_blockchain_peer():
                 blockchain_tx_message = self.node.message_converter.bx_tx_to_tx(msg)
                 transaction_feed_stats_service.log_new_transaction(tx_hash)
-                sent = self.node.send_transaction_to_node(blockchain_tx_message)
+
+                sent = self.node.broadcast_transactions_to_nodes(blockchain_tx_message, self)
                 if sent:
                     tx_stats.add_tx_by_hash_event(
                         tx_hash,
@@ -221,6 +222,7 @@ class AbstractRelayConnection(InternalNodeConnection["AbstractGatewayNode"]):
                         network_num,
                         short_id
                     )
+                    gateway_bdn_performance_stats_service.log_tx_sent_to_nodes()
                 else:
                     gateway_transaction_stats_service.log_dropped_transaction_from_relay()
 
@@ -263,7 +265,7 @@ class AbstractRelayConnection(InternalNodeConnection["AbstractGatewayNode"]):
             map(lambda x: x.short_id, transactions),
             TransactionStatEventType.TX_UNKNOWN_SHORT_IDS_REPLY_RECEIVED_BY_GATEWAY_FROM_RELAY,
             network_num=self.node.network_num,
-            peer=stats_format.connection(self),
+            peers=[self],
             found_tx_hashes=map(lambda x: convert.bytes_to_hex(x.hash.binary),
                                 transactions
                                 )
@@ -278,7 +280,7 @@ class AbstractRelayConnection(InternalNodeConnection["AbstractGatewayNode"]):
                 TransactionStatEventType.TX_UNKNOWN_TRANSACTION_RECEIVED_BY_GATEWAY_FROM_RELAY,
                 self.node.network_num,
                 short_id,
-                peer=stats_format.connection(self)
+                peers=[self]
             )
 
         self.node.block_processing_service.retry_broadcast_recovered_blocks(self)
@@ -295,6 +297,7 @@ class AbstractRelayConnection(InternalNodeConnection["AbstractGatewayNode"]):
             msg.block_hash(),
             BlockStatEventType.ENC_BLOCK_COMPRESSED_TXS_RECEIVED_BY_GATEWAY,
             network_num=msg.network_num(),
+            peers=[self],
             more_info="{}. processing time: {}".format(
                 stats_format.connection(self),
                 stats_format.timespan(start_time, time.time())
@@ -341,14 +344,15 @@ class AbstractRelayConnection(InternalNodeConnection["AbstractGatewayNode"]):
 
     def send_bdn_performance_stats(self, bdn_stats_interval: GatewayBdnPerformanceStatInterval):
         memory_utilization_mb = int(memory_utils.get_app_memory_usage() / constants.BYTE_TO_MB)
+        bdn_stats_per_node = bdn_stats_interval.blockchain_node_to_bdn_stats
+        if bdn_stats_per_node is None:
+            bdn_stats_per_node = {}
+
         msg_to_send = BdnPerformanceStatsMessage(
             bdn_stats_interval.start_time,
             bdn_stats_interval.end_time,
-            bdn_stats_interval.new_blocks_received_from_blockchain_node,
-            bdn_stats_interval.new_blocks_received_from_bdn,
-            bdn_stats_interval.new_tx_received_from_blockchain_node,
-            bdn_stats_interval.new_tx_received_from_bdn,
-            memory_utilization_mb
+            memory_utilization_mb,
+            bdn_stats_per_node
         )
         self.enqueue_msg(msg_to_send)
 
@@ -366,7 +370,7 @@ class AbstractRelayConnection(InternalNodeConnection["AbstractGatewayNode"]):
                     2 * constants.MIN_SLEEP_TIMEOUT, constants.MIN_SLEEP_TIMEOUT, status_log.update_alarm_callback,
                     self.node.connection_pool, self.node.opts.use_extensions, self.node.opts.source_version,
                     self.node.opts.external_ip, self.node.opts.continent, self.node.opts.country,
-                    self.node.opts.should_update_source_version, self.node.account_id,
+                    self.node.opts.should_update_source_version, self.node.blockchain_peers, self.node.account_id,
                     self.node.quota_level
                 )
         elif (
@@ -404,12 +408,43 @@ class AbstractRelayConnection(InternalNodeConnection["AbstractGatewayNode"]):
             done_callback=self._process_blockchain_network_from_sdn
         )
 
-    def publish_new_transaction(self, tx_hash: Sha256Hash, tx_contents: memoryview) -> None:
+    def publish_new_transaction(
+        self, tx_hash: Sha256Hash, tx_contents: memoryview, local_region: bool
+    ) -> None:
         self.node.feed_manager.publish_to_feed(
-            NewTransactionFeed.NAME,
-            RawTransactionFeedEntry(tx_hash, tx_contents)
+            FeedKey(NewTransactionFeed.NAME),
+            RawTransactionFeedEntry(tx_hash, tx_contents, local_region=local_region)
         )
 
+    def on_connection_established(self):
+        super(AbstractRelayConnection, self).on_connection_established()
+
+        if self.is_relay_connection() and self.node.opts.split_relays:
+            self.node.alarm_queue.register_alarm(gateway_constants.CHECK_RELAY_CONNECTIONS_DELAY_S,
+                                                 self._check_matching_relay_connection)
+
+    def _check_matching_relay_connection(self):
+        self.log_debug("Verifying that matching relay connection has been established.")
+
+        matching_relay_ip = self.peer_ip
+        if self.CONNECTION_TYPE == ConnectionType.RELAY_TRANSACTION:
+            matching_relay_port = self.peer_port - 1
+        else:
+            matching_relay_port = self.peer_port + 1
+
+        if self.node.connection_pool.has_connection(matching_relay_ip, matching_relay_port):
+            matching_connection = self.node.connection_pool.by_ipport[(matching_relay_ip, matching_relay_port)]
+            if matching_connection.is_active():
+                self.log_debug("Matching connection check for relay connection {} is successful. "
+                               "Matching connection is {}", self, matching_connection)
+                return
+
+        self.log_debug("Closing relay connection {} because matching relay connection is not available.", self)
+        if self.CONNECTION_TYPE == ConnectionType.RELAY_TRANSACTION:
+            self.node.remove_relay_transaction_peer(self.peer_ip, self.peer_port, True)
+        else:
+            self.node.remove_relay_peer(self.peer_ip, self.peer_port)
+
     def _process_blockchain_network_from_sdn(self, get_blockchain_network_future: Future):
         try:
             updated_blockchain_network = get_blockchain_network_future.result()
@@ -419,4 +454,4 @@ class AbstractRelayConnection(InternalNodeConnection["AbstractGatewayNode"]):
             self.node.opts.blockchain_networks[self.node.network_num] = updated_blockchain_network
             self.node.update_node_settings_from_blockchain_network(updated_blockchain_network)
         except Exception as e:
-            self.log_info("Got {} when trying to read blockchain network from sdn", e)
\ No newline at end of file
+            self.log_info("Got {} when trying to read blockchain network from sdn", e)
