@@ -6,48 +6,57 @@ from bxcommon import constants
 from bxcommon.connections.abstract_connection import AbstractConnection
 from bxcommon.connections.connection_state import ConnectionState
 from bxcommon.connections.connection_type import ConnectionType
+from bxcommon.feed.feed import FeedKey
+from bxcommon.messages.abstract_block_message import AbstractBlockMessage
 from bxcommon.messages.abstract_message import AbstractMessage
 from bxcommon.messages.eth.serializers.transaction import Transaction
+from bxcommon.models.blockchain_peer_info import BlockchainPeerInfo
 from bxcommon.network.abstract_socket_connection_protocol import AbstractSocketConnectionProtocol
 from bxcommon.network.ip_endpoint import IpEndpoint
 from bxcommon.network.peer_info import ConnectionPeerInfo
 from bxcommon.network.transport_layer_protocol import TransportLayerProtocol
 from bxcommon.rpc import rpc_constants
+from bxcommon.feed.feed_source import FeedSource
 from bxcommon.utils import convert
+from bxcommon.utils.expiring_dict import ExpiringDict
+from bxcommon.utils.blockchain_utils.eth import crypto_utils, eth_common_constants, eth_common_utils
 from bxcommon.utils.object_hash import Sha256Hash
 from bxcommon.utils.stats.block_stat_event_type import BlockStatEventType
 from bxcommon.utils.stats.block_statistics_service import block_stats
-from bxgateway.feed.feed_source import FeedSource
-from bxgateway.messages.eth.internal_eth_block_info import InternalEthBlockInfo
+from bxcommon.utils.stats.transaction_stat_event_type import TransactionStatEventType
+from bxcommon.utils.stats.transaction_statistics_service import tx_stats
+from bxgateway import log_messages, gateway_constants
 from bxgateway.connections.abstract_gateway_blockchain_connection import AbstractGatewayBlockchainConnection
 from bxgateway.connections.abstract_gateway_node import AbstractGatewayNode
 from bxgateway.connections.abstract_relay_connection import AbstractRelayConnection
+from bxgateway.connections.eth.eth_base_connection_protocol import EthConnectionProtocolStatus
 from bxgateway.connections.eth.eth_node_connection import EthNodeConnection
 from bxgateway.connections.eth.eth_node_discovery_connection import EthNodeDiscoveryConnection
 from bxgateway.connections.eth.eth_relay_connection import EthRelayConnection
 from bxgateway.connections.eth.eth_remote_connection import EthRemoteConnection
 from bxgateway.connections.gateway_connection import GatewayConnection
 from bxgateway.feed.eth.eth_new_block_feed import EthNewBlockFeed
-from bxgateway.feed.eth.eth_new_transaction_feed import EthNewTransactionFeed
-from bxgateway.feed.eth.eth_pending_transaction_feed import EthPendingTransactionFeed
+from bxcommon.feed.eth.eth_new_transaction_feed import EthNewTransactionFeed
 from bxgateway.feed.eth.eth_on_block_feed import EthOnBlockFeed, EventNotification
+from bxcommon.feed.eth.eth_pending_transaction_feed import EthPendingTransactionFeed
 from bxgateway.feed.eth.eth_raw_block import EthRawBlock
+from bxgateway.feed.eth.eth_transaction_receipts_feed import EthTransactionReceiptsFeed
 from bxgateway.messages.eth import eth_message_converter_factory as converter_factory
+from bxgateway.messages.eth.internal_eth_block_info import InternalEthBlockInfo
 from bxgateway.messages.eth.new_block_parts import NewBlockParts
 from bxgateway.messages.eth.protocol.transactions_eth_protocol_message import \
     TransactionsEthProtocolMessage
-from bxgateway.rpc.external.eth_ws_subscriber import EthWsSubscriber
+from bxgateway.rpc.external.eth_ws_proxy_publisher import EthWsProxyPublisher
 from bxgateway.services.abstract_block_cleanup_service import AbstractBlockCleanupService
 from bxgateway.services.eth.eth_block_processing_service import EthBlockProcessingService
 from bxgateway.services.eth.eth_block_queuing_service import EthBlockQueuingService
 from bxgateway.services.eth.eth_normal_block_cleanup_service import EthNormalBlockCleanupService
 from bxgateway.testing.eth_lossy_relay_connection import EthLossyRelayConnection
 from bxgateway.testing.test_modes import TestModes
+from bxgateway.utils.interval_minimum import IntervalMinimum
 from bxgateway.utils.running_average import RunningAverage
-from bxcommon.utils.blockchain_utils.eth import crypto_utils, eth_common_constants, eth_common_utils
 from bxgateway.utils.stats.eth.eth_gateway_stats_service import eth_gateway_stats_service
 from bxgateway.utils.stats.eth_on_block_feed_stats_service import eth_on_block_feed_stats_service
-from bxgateway import log_messages, gateway_constants
 from bxutils import logging
 from bxutils.services.node_ssl_service import NodeSSLService
 
@@ -78,14 +87,18 @@ class EthGatewayNode(AbstractGatewayNode):
     }
 
     def __init__(self, opts, node_ssl_service: NodeSSLService) -> None:
-        super(EthGatewayNode, self).__init__(opts, node_ssl_service, eth_common_constants.TRACKED_BLOCK_CLEANUP_INTERVAL_S)
+        super(EthGatewayNode, self).__init__(opts, node_ssl_service,
+                                             eth_common_constants.TRACKED_BLOCK_CLEANUP_INTERVAL_S)
+
+        self.remote_blockchain_protocol_version = eth_common_constants.ETH_PROTOCOL_VERSION
+        self.remote_blockchain_connection_established = False
 
         self._node_public_key = None
         self._remote_public_key = None
 
         if opts.node_public_key is not None:
             self._node_public_key = convert.hex_to_bytes(opts.node_public_key)
-        else:
+        elif opts.blockchain_peers is None:
             raise RuntimeError(
                 "128 digit public key must be included with command-line specified blockchain peer.")
         if opts.remote_blockchain_peer is not None:
@@ -96,7 +109,11 @@ class EthGatewayNode(AbstractGatewayNode):
                 self._remote_public_key = convert.hex_to_bytes(opts.remote_public_key)
 
         self.block_processing_service: EthBlockProcessingService = EthBlockProcessingService(self)
-        self.block_queuing_service: EthBlockQueuingService = EthBlockQueuingService(self)
+        self.block_parts_storage: ExpiringDict[Sha256Hash, NewBlockParts] = ExpiringDict(
+            self.alarm_queue,
+            gateway_constants.MAX_BLOCK_CACHE_TIME_S,
+            "eth_block_queue_parts",
+        )
 
         # List of know total difficulties, tuples of values (block hash, total difficulty)
         self._last_known_difficulties = deque(maxlen=eth_common_constants.LAST_KNOWN_TOTAL_DIFFICULTIES_MAX_COUNT)
@@ -111,11 +128,12 @@ class EthGatewayNode(AbstractGatewayNode):
         self.init_eth_on_block_feed_stat_logging()
 
         self.message_converter = converter_factory.create_eth_message_converter(self.opts)
-        self.eth_ws_subscriber = EthWsSubscriber(opts.eth_ws_uri, self.feed_manager, self._tx_service, self)
+        self.eth_ws_proxy_publisher = EthWsProxyPublisher(opts.eth_ws_uri, self.feed_manager, self._tx_service, self)
         if self.opts.ws and not self.opts.eth_ws_uri:
             logger.warning(log_messages.ETH_WS_SUBSCRIBER_NOT_STARTED)
 
-        self.average_gas_price = RunningAverage(gateway_constants.ETH_GAS_RUNNING_AVERAGE_SIZE)
+        self.average_block_gas_price = RunningAverage(gateway_constants.ETH_GAS_RUNNING_AVERAGE_SIZE)
+        self.min_tx_from_node_gas_price = IntervalMinimum(gateway_constants.ETH_MIN_GAS_INTERVAL_S, self.alarm_queue)
 
         logger.info("Gateway enode url: {}", self.get_enode())
 
@@ -141,8 +159,11 @@ class EthGatewayNode(AbstractGatewayNode):
     ) -> AbstractGatewayBlockchainConnection:
         return EthRemoteConnection(socket_connection, self)
 
-    def build_block_queuing_service(self) -> EthBlockQueuingService:
-        return EthBlockQueuingService(self)
+    def build_block_queuing_service(
+        self,
+        connection: AbstractGatewayBlockchainConnection
+    ) -> EthBlockQueuingService:
+        return EthBlockQueuingService(self, connection)
 
     def build_block_cleanup_service(self) -> AbstractBlockCleanupService:
         if self.opts.use_extensions:
@@ -169,11 +190,12 @@ class EthGatewayNode(AbstractGatewayNode):
             ))
 
         local_protocol = TransportLayerProtocol.UDP if self._is_in_local_discovery() else TransportLayerProtocol.TCP
-        peers.append(ConnectionPeerInfo(
-            IpEndpoint(self.opts.blockchain_ip, self.opts.blockchain_port),
-            ConnectionType.BLOCKCHAIN_NODE,
-            local_protocol
-        ))
+        for blockchain_peer in self.blockchain_peers:
+            peers.append(ConnectionPeerInfo(
+                IpEndpoint(blockchain_peer.ip, blockchain_peer.port),
+                ConnectionType.BLOCKCHAIN_NODE,
+                local_protocol
+            ))
 
         if self.remote_blockchain_ip is not None and self.remote_blockchain_port is not None:
             remote_protocol = TransportLayerProtocol.UDP if self._is_in_remote_discovery() else \
@@ -231,8 +253,19 @@ class EthGatewayNode(AbstractGatewayNode):
             remote_blockchain_ip, remote_blockchain_port, ConnectionType.REMOTE_BLOCKCHAIN_NODE
         )
 
-    def get_node_public_key(self) -> bytes:
-        return self._node_public_key
+    def get_node_public_key(self, ip: str, port: int) -> bytes:
+        node_public_key = None
+        for blockchain_peer in self.blockchain_peers:
+            if blockchain_peer.ip == ip and blockchain_peer.port == port:
+                node_public_key = blockchain_peer.node_public_key
+                break
+
+        if not node_public_key:
+            raise RuntimeError(
+                f"128 digit public key must be included with for blockchain peer ip {ip} and port {port}."
+            )
+
+        return convert.hex_to_bytes(node_public_key)
 
     def get_remote_public_key(self) -> bytes:
         return self._remote_public_key
@@ -272,7 +305,7 @@ class EthGatewayNode(AbstractGatewayNode):
                     BlockStatEventType.REMOTE_BLOCK_REQUESTED_BY_GATEWAY,
                     network_num=self.network_num,
                     more_info=f"Protocol: {self.opts.blockchain_protocol}, "
-                              f"Network: {self.opts.blockchain_network}"
+                    f"Network: {self.opts.blockchain_network}"
                 )
             self._requested_remote_blocks_queue.append(block_hashes)
 
@@ -282,7 +315,7 @@ class EthGatewayNode(AbstractGatewayNode):
 
             if len(expected_blocks) != blocks_count:
                 logger.warning(log_messages.BLOCK_COUNT_MISMATCH,
-                    blocks_count, len(expected_blocks))
+                               blocks_count, len(expected_blocks))
                 self._skip_remote_block_requests_stats_count = len(self._requested_remote_blocks_queue) * 2
                 self._requested_remote_blocks_queue.clear()
                 return
@@ -297,12 +330,77 @@ class EthGatewayNode(AbstractGatewayNode):
         else:
             logger.warning(log_messages.UNEXPECTED_BLOCKS)
 
+    def _should_decrease_version_number(
+        self, connection_state: ConnectionState, connection_status: EthConnectionProtocolStatus
+    ) -> bool:
+        return (
+            ConnectionState.ESTABLISHED not in connection_state
+            and connection_status.auth_message_sent
+            and not connection_status.auth_message_received
+            and not connection_status.auth_ack_message_sent
+            and connection_status.auth_ack_message_received
+            and connection_status.hello_message_sent
+            and connection_status.hello_message_received
+            and not connection_status.status_message_sent
+            and not connection_status.status_message_received
+            and connection_status.disconnect_message_received
+            and connection_status.disconnect_reason == 3
+        )
+
+    def _get_new_protocol_version(self, connection, peer_version) -> int:
+        for index, version in enumerate(eth_common_constants.SUPPORTED_ETH_PROTOCOL_VERSION):
+            if (
+                version == peer_version
+                and index < len(eth_common_constants.SUPPORTED_ETH_PROTOCOL_VERSION) - 1
+            ):
+                new_version = eth_common_constants.SUPPORTED_ETH_PROTOCOL_VERSION[index + 1]
+                connection.log_debug(
+                    "Failed to connect with version {}, try to reconnect with version {}",
+                    version, new_version
+                )
+                return new_version
+
+        return peer_version
+
+    def _should_log_closed_connection(self, connection: AbstractConnection) -> bool:
+        if isinstance(connection, EthNodeConnection):
+            connection_status = connection.connection_protocol.connection_status
+
+            if (
+                self._should_decrease_version_number(connection.state, connection_status)
+                and connection.CONNECTION_TYPE == ConnectionType.BLOCKCHAIN_NODE
+            ):
+                for peer in self.blockchain_peers:
+                    if (
+                        peer == BlockchainPeerInfo(connection.peer_ip, connection.peer_port)
+                        and not peer.connection_established
+                    ):
+                        peer.blockchain_protocol_version = self._get_new_protocol_version(
+                            connection, peer.blockchain_protocol_version
+                        )
+                        return False
+
+        elif isinstance(connection, EthRemoteConnection):
+            connection_status = connection.connection_protocol.connection_status
+
+            if (
+                self._should_decrease_version_number(connection.state, connection_status)
+                and connection.CONNECTION_TYPE == ConnectionType.REMOTE_BLOCKCHAIN_NODE
+                and not self.remote_blockchain_connection_established
+            ):
+                self.remote_blockchain_protocol_version = \
+                    self._get_new_protocol_version(
+                        connection, self.remote_blockchain_protocol_version
+                    )
+                return False
+
+        return True
+
     def log_closed_connection(self, connection: AbstractConnection) -> None:
         if isinstance(connection, EthNodeConnection):
             # pyre-fixme[22]: The cast is redundant.
             eth_node_connection = cast(EthNodeConnection, connection)
             connection_status = connection.connection_protocol.connection_status
-
             if ConnectionState.INITIALIZED not in eth_node_connection.state:
                 logger.info("Failed to connect to Ethereum node. Verify that provided ip address ({}) and port ({}) "
                             "are correct. Verify that firewall port is open. Connection details: {}.",
@@ -327,6 +425,7 @@ class EthGatewayNode(AbstractGatewayNode):
                             eth_node_connection)
             else:
                 super(EthGatewayNode, self).log_closed_connection(connection)
+
         elif isinstance(connection, GatewayConnection):
             if ConnectionState.ESTABLISHED not in connection.state:
                 logger.debug("Failed to connect to: {}.", connection)
@@ -341,43 +440,77 @@ class EthGatewayNode(AbstractGatewayNode):
 
     def init_eth_on_block_feed_stat_logging(self) -> None:
         eth_on_block_feed_stats_service.set_node(self)
-        self.alarm_queue.register_alarm(eth_on_block_feed_stats_service.interval, eth_on_block_feed_stats_service.flush_info)
+        self.alarm_queue.register_alarm(eth_on_block_feed_stats_service.interval,
+                                        eth_on_block_feed_stats_service.flush_info)
 
     def init_live_feeds(self) -> None:
         self.feed_manager.register_feed(EthNewTransactionFeed())
         self.feed_manager.register_feed(EthPendingTransactionFeed(self.alarm_queue))
         self.feed_manager.register_feed(EthOnBlockFeed(self))
         self.feed_manager.register_feed(EthNewBlockFeed(self))
+        self.feed_manager.register_feed(EthTransactionReceiptsFeed(self))
 
     def on_new_subscriber_request(self) -> None:
-        if self.opts.eth_ws_uri and not self.eth_ws_subscriber.running:
-            asyncio.create_task(self.eth_ws_subscriber.revive())
+        if self.opts.eth_ws_uri and not self.eth_ws_proxy_publisher.running:
+            asyncio.create_task(self.eth_ws_proxy_publisher.revive())
 
     def on_transactions_in_block(self, transactions: List[Transaction]) -> None:
         for transaction in transactions:
-            self.average_gas_price.add_value(transaction.gas_price)
+            self.average_block_gas_price.add_value(transaction.gas_price)
 
-    def send_transaction_to_node(self, msg: AbstractMessage) -> bool:
+    def broadcast_transactions_to_nodes(
+        self, msg: AbstractMessage, broadcasting_conn: Optional[AbstractConnection]
+    ) -> bool:
         msg = cast(TransactionsEthProtocolMessage, msg)
+
         if self.opts.filter_txs_factor > 0:
+            average_block_gas_filter = self.average_block_gas_price.average * self.opts.filter_txs_factor
+        else:
+            average_block_gas_filter = 0
+        min_gas_price_from_node = self.min_tx_from_node_gas_price.current_minimum
+
+        gas_price_filter = max(average_block_gas_filter, min_gas_price_from_node)
+
+        if gas_price_filter > 0:
             assert len(msg.get_transactions()) == 1
             transaction = msg.get_transactions()[0]
 
-            if (float(transaction.gas_price) < self.average_gas_price.average * self.opts.filter_txs_factor):
-                logger.debug(
-                    "Skipping sending transaction {} with gas price: {}. Average was {}",
+            gas_price = float(transaction.gas_price)
+
+            if gas_price < gas_price_filter:
+                logger.trace(
+                    "Skipping sending transaction {} with gas price: {}. Average was {}. Minimum from node was {}.",
                     transaction.hash(),
                     float(transaction.gas_price),
-                    self.average_gas_price.average
+                    average_block_gas_filter,
+                    min_gas_price_from_node
+                )
+                tx_stats.add_tx_by_hash_event(
+                    transaction.hash(),
+                    TransactionStatEventType.TX_FROM_BDN_IGNORE_LOW_GAS_PRICE,
+                    self.network_num,
+                    peers=[broadcasting_conn],
+                    more_info="Tx gas price {}. Average block gas price: {}. Node min gas price {}."
+                        .format(gas_price, average_block_gas_filter, min_gas_price_from_node)
                 )
                 return False
 
-        return super().send_transaction_to_node(msg)
+        return super().broadcast_transactions_to_nodes(msg, broadcasting_conn)
 
     def get_enode(self) -> str:
         return \
             f"enode://{convert.bytes_to_hex(self.get_public_key())}@{self.opts.external_ip}:{self.opts.non_ssl_port}"
 
+    def on_block_received_from_bdn(
+        self, block_hash: Sha256Hash, block_message: AbstractBlockMessage
+    ) -> None:
+        super().on_block_received_from_bdn(block_hash, block_message)
+
+        node_conn = cast(EthNodeConnection, self.get_any_active_blockchain_connection())
+        if node_conn is not None:
+            eth_connection_protocol = node_conn.connection_protocol
+            eth_connection_protocol.pending_new_block_parts.remove_item(block_hash)
+
     # pyre-fixme[14]: Inconsistent override:
     # Parameter of type InternalEthBlockInfo is not a supertype of AbstractBlockMessage
     def publish_block(
@@ -401,13 +534,15 @@ class EthGatewayNode(AbstractGatewayNode):
                 self._get_block_message_lazy(block_message, block_hash)
             )
             self._publish_block_to_on_block_feed(raw_block)
-            self._publish_block_to_new_block_feed(raw_block)
+            self._publish_block_to_transaction_receipts_feed(raw_block)
+            if block_message is not None:
+                self._publish_block_to_new_block_feed(raw_block)
 
     async def init(self) -> None:
         await super().init()
         try:
             await asyncio.wait_for(
-                self.eth_ws_subscriber.start(), rpc_constants.RPC_SERVER_INIT_TIMEOUT_S
+                self.eth_ws_proxy_publisher.start(), rpc_constants.RPC_SERVER_INIT_TIMEOUT_S
             )
         except Exception as e:
             logger.error(log_messages.ETH_WS_INITIALIZATION_FAIL, e)
@@ -416,7 +551,7 @@ class EthGatewayNode(AbstractGatewayNode):
     async def close(self) -> None:
         try:
             await asyncio.wait_for(
-                self.eth_ws_subscriber.stop(),
+                self.eth_ws_proxy_publisher.stop(),
                 rpc_constants.RPC_SERVER_STOP_TIMEOUT_S
             )
         except Exception as e:
@@ -447,17 +582,21 @@ class EthGatewayNode(AbstractGatewayNode):
         if block_message:
             yield block_message
         else:
-            block_parts = self.block_queuing_service.get_block_parts(block_hash)
-            if block_parts:
-                block_message = InternalEthBlockInfo.from_new_block_parts(block_parts)
-                assert block_message is not None
-                yield block_message
+            block_queuing_service = cast(
+                EthBlockQueuingService,
+                self.block_queuing_service_manager.get_designated_block_queuing_service()
+            )
+            if block_queuing_service is not None:
+                block_parts = block_queuing_service.get_block_parts(block_hash)
+                if block_parts:
+                    block_message = InternalEthBlockInfo.from_new_block_parts(block_parts)
+                    yield block_message
 
     def _publish_block_to_new_block_feed(
         self,
         raw_block: EthRawBlock,
     ) -> None:
-        self.feed_manager.publish_to_feed(EthNewBlockFeed.NAME, raw_block)
+        self.feed_manager.publish_to_feed(FeedKey(EthNewBlockFeed.NAME), raw_block)
 
     def _publish_block_to_on_block_feed(
         self,
@@ -465,9 +604,17 @@ class EthGatewayNode(AbstractGatewayNode):
     ) -> None:
         if raw_block.source in {FeedSource.BLOCKCHAIN_RPC}:
             self.feed_manager.publish_to_feed(
-                EthOnBlockFeed.NAME, EventNotification(raw_block.block_number)
+                FeedKey(EthOnBlockFeed.NAME), EventNotification(raw_block.block_number)
             )
 
+    def _publish_block_to_transaction_receipts_feed(
+        self,
+        raw_block: EthRawBlock
+    ):
+        self.feed_manager.publish_to_feed(
+            FeedKey(EthTransactionReceiptsFeed.NAME), raw_block
+        )
+
     def is_gas_price_above_min_network_fee(self, transaction_contents: Union[memoryview, bytearray]) -> bool:
         gas_price = eth_common_utils.raw_tx_gas_price(memoryview(transaction_contents), 0)
         if gas_price >= self.get_blockchain_network().min_tx_network_fee:
