@@ -12,7 +12,7 @@ class EthNodeConnection(EthBaseConnection):
     def __init__(self, sock: AbstractSocketConnectionProtocol, node: "EthGatewayNode"):
         super(EthNodeConnection, self).__init__(sock, node)
 
-        node_public_key = self.node.get_node_public_key()
+        node_public_key = self.node.get_node_public_key(self.peer_ip, self.peer_port)
         is_handshake_initiator = node_public_key is not None
         private_key = self.node.get_private_key()
         self.connection_protocol = EthNodeConnectionProtocol(
