@@ -1,22 +1,22 @@
 import time
 from collections import deque
-from typing import List, Deque
+from typing import List, Deque, cast, Union
 
+from bxcommon.feed.feed import FeedKey
 from bxcommon.messages.abstract_message import AbstractMessage
 from bxcommon.utils import convert
 from bxcommon.utils.blockchain_utils.eth import eth_common_utils, eth_common_constants
 from bxcommon.utils.expiring_dict import ExpiringDict
 from bxcommon.utils.object_hash import Sha256Hash, NULL_SHA256_HASH
-from bxcommon.utils.stats import stats_format
 from bxcommon.utils.stats.block_stat_event_type import BlockStatEventType
 from bxcommon.utils.stats.block_statistics_service import block_stats
+from bxcommon.feed.feed_source import FeedSource
 from bxgateway import log_messages
 from bxgateway.connections.eth.eth_base_connection_protocol import EthBaseConnectionProtocol
 from bxgateway.eth_exceptions import CipherNotInitializedError
-from bxgateway.feed.eth.eth_new_transaction_feed import EthNewTransactionFeed
-from bxgateway.feed.eth.eth_pending_transaction_feed import EthPendingTransactionFeed
-from bxgateway.feed.eth.eth_raw_transaction import EthRawTransaction
-from bxgateway.feed.new_transaction_feed import FeedSource
+from bxcommon.feed.eth.eth_new_transaction_feed import EthNewTransactionFeed
+from bxcommon.feed.eth.eth_pending_transaction_feed import EthPendingTransactionFeed
+from bxcommon.feed.eth.eth_raw_transaction import EthRawTransaction
 from bxgateway.messages.eth.internal_eth_block_info import InternalEthBlockInfo
 from bxgateway.messages.eth.new_block_parts import NewBlockParts
 from bxgateway.messages.eth.protocol.block_bodies_eth_protocol_message import BlockBodiesEthProtocolMessage
@@ -28,9 +28,15 @@ from bxgateway.messages.eth.protocol.get_receipts_eth_protocol_message import Ge
 from bxgateway.messages.eth.protocol.new_block_eth_protocol_message import NewBlockEthProtocolMessage
 from bxgateway.messages.eth.protocol.new_block_hashes_eth_protocol_message import NewBlockHashesEthProtocolMessage
 from bxgateway.messages.eth.protocol.status_eth_protocol_message import StatusEthProtocolMessage
+from bxgateway.messages.eth.protocol.status_eth_protocol_message_v63 import StatusEthProtocolMessageV63
 from bxgateway.messages.eth.protocol.transactions_eth_protocol_message import \
     TransactionsEthProtocolMessage
+from bxgateway.services.eth.eth_block_queuing_service import EthBlockQueuingService
+from bxgateway.services.gateway_transaction_service import ProcessTransactionMessageFromNodeResult
+from bxgateway.utils.stats.gateway_bdn_performance_stats_service import \
+    gateway_bdn_performance_stats_service
 from bxgateway.utils.stats.gateway_transaction_stats_service import gateway_transaction_stats_service
+from bxgateway.utils.stats.transaction_feed_stats_service import transaction_feed_stats_service
 from bxutils import logging
 
 logger = logging.get_logger(__name__)
@@ -56,7 +62,7 @@ class EthNodeConnectionProtocol(EthBaseConnectionProtocol):
         self.waiting_checkpoint_headers_request = True
 
         # uses block hash as a key, and NewBlockParts structure as value
-        self._pending_new_blocks_parts: ExpiringDict[
+        self.pending_new_block_parts: ExpiringDict[
             Sha256Hash, NewBlockParts
         ] = ExpiringDict(
             self.node.alarm_queue,
@@ -86,8 +92,9 @@ class EthNodeConnectionProtocol(EthBaseConnectionProtocol):
         )
         self._connection_established_time = 0.0
 
-    def msg_status(self, msg: StatusEthProtocolMessage):
+    def msg_status(self, msg: Union[StatusEthProtocolMessage, StatusEthProtocolMessageV63]):
         super(EthNodeConnectionProtocol, self).msg_status(msg)
+
         self.connection.on_connection_established()
 
         self.connection.schedule_pings()
@@ -108,6 +115,15 @@ class EthNodeConnectionProtocol(EthBaseConnectionProtocol):
         else:
             super().msg_tx(msg)
 
+    def msg_tx_after_tx_service_process_complete(self, process_result: List[ProcessTransactionMessageFromNodeResult]):
+        # calculate minimal tx gas price only if transaction validation is enabled
+        if not self.node.opts.transaction_validation:
+            return
+
+        for result in process_result:
+            fee = eth_common_utils.raw_tx_gas_price(memoryview(result.transaction_contents), 0)
+            self.node.min_tx_from_node_gas_price.add(fee)
+
     # pyre-fixme[14]: `msg_block` overrides method defined in
     #  `AbstractBlockchainConnectionProtocol` inconsistently.
     def msg_block(self, msg: NewBlockEthProtocolMessage) -> None:
@@ -139,9 +155,13 @@ class EthNodeConnectionProtocol(EthBaseConnectionProtocol):
                 ),
                 block_height=block_number,
             )
+            gateway_bdn_performance_stats_service.log_block_message_from_blockchain_node(
+                self.connection.endpoint,
+                False
+            )
 
             if block_hash in self.node.blocks_seen.contents:
-                self.node.on_block_seen_by_blockchain_node(block_hash, block_number=block_number)
+                self.node.on_block_seen_by_blockchain_node(block_hash, self.connection, block_number=block_number)
                 block_stats.add_block_event_by_block_hash(
                     block_hash,
                     BlockStatEventType.BLOCK_RECEIVED_FROM_BLOCKCHAIN_NODE_IGNORE_SEEN,
@@ -154,11 +174,7 @@ class EthNodeConnectionProtocol(EthBaseConnectionProtocol):
                 )
                 continue
 
-            recovery_cancelled = self.node.on_block_seen_by_blockchain_node(
-                block_hash, block_number=block_number
-            )
-            if recovery_cancelled:
-                continue
+            self.node.on_block_seen_by_blockchain_node(block_hash, self.connection, block_number=block_number)
 
             self.node.track_block_from_node_handling_started(block_hash)
             block_hash_number_pairs.append((block_hash, block_number))
@@ -173,10 +189,8 @@ class EthNodeConnectionProtocol(EthBaseConnectionProtocol):
 
         for block_hash, block_number in block_hash_number_pairs:
             # pyre-fixme[6]: Expected `memoryview` for 1st param but got `None`.
-            self._pending_new_blocks_parts.add(block_hash, NewBlockParts(None, None, block_number))
-            self.node.send_msg_to_node(
-                GetBlockHeadersEthProtocolMessage(None, block_hash.binary, 1, 0, False)
-            )
+            self.pending_new_block_parts.add(block_hash, NewBlockParts(None, None, block_number))
+            self.connection.enqueue_msg(GetBlockHeadersEthProtocolMessage(None, block_hash.binary, 1, 0, 0))
 
         self.request_block_body([block_hash for block_hash, _ in block_hash_number_pairs])
 
@@ -185,25 +199,34 @@ class EthNodeConnectionProtocol(EthBaseConnectionProtocol):
             None,
             block_hashes=[bytes(blk.binary) for blk in block_hashes]
         )
-        self.node.send_msg_to_node(block_request_message)
+        self.connection.enqueue_msg(block_request_message)
         self._block_bodies_requests.append(block_hashes)
 
     def msg_get_block_headers(self, msg: GetBlockHeadersEthProtocolMessage):
         if self._waiting_checkpoint_headers_request:
             super(EthNodeConnectionProtocol, self).msg_get_block_headers(msg)
         else:
+            block_queuing_service = cast(
+                EthBlockQueuingService,
+                self.node.block_queuing_service_manager.get_block_queuing_service(self.connection)
+            )
             processed = self.node.block_processing_service.try_process_get_block_headers_request(
-                msg
+                msg,
+                block_queuing_service
             )
             if not processed:
-                self.msg_proxy_request(msg)
+                self.msg_proxy_request(msg, self.connection)
 
     def msg_get_block_bodies(self, msg: GetBlockBodiesEthProtocolMessage):
-        processed = self.node.block_processing_service.try_process_get_block_bodies_request(msg)
+        block_queuing_service = cast(
+            EthBlockQueuingService,
+            self.node.block_queuing_service_manager.get_block_queuing_service(self.connection)
+        )
+        processed = self.node.block_processing_service.try_process_get_block_bodies_request(msg, block_queuing_service)
 
         if not processed:
             self.node.log_requested_remote_blocks(msg.get_block_hashes())
-            self.msg_proxy_request(msg)
+            self.msg_proxy_request(msg, self.connection)
 
     def msg_block_headers(self, msg: BlockHeadersEthProtocolMessage):
         if not self.node.should_process_block_hash():
@@ -211,14 +234,14 @@ class EthNodeConnectionProtocol(EthBaseConnectionProtocol):
 
         block_headers = msg.get_block_headers()
 
-        if self._pending_new_blocks_parts.contents and len(block_headers) == 1:
+        if self.pending_new_block_parts.contents and len(block_headers) == 1:
             header_bytes = msg.get_block_headers_bytes()[0]
             block_hash_bytes = eth_common_utils.keccak_hash(header_bytes)
             block_hash = Sha256Hash(block_hash_bytes)
 
-            if block_hash in self._pending_new_blocks_parts.contents:
+            if block_hash in self.pending_new_block_parts.contents:
                 logger.debug("Received block header for new block {}", convert.bytes_to_hex(block_hash.binary))
-                self._pending_new_blocks_parts.contents[block_hash].block_header_bytes = header_bytes
+                self.pending_new_block_parts.contents[block_hash].block_header_bytes = header_bytes
                 self._check_pending_new_block(block_hash)
                 self._process_ready_new_blocks()
                 return
@@ -227,7 +250,24 @@ class EthNodeConnectionProtocol(EthBaseConnectionProtocol):
             block_hashes = [blk.hash_object() for blk in block_headers]
             block_hashes.insert(0, Sha256Hash(block_headers[0].prev_hash))
             self.node.block_cleanup_service.mark_blocks_and_request_cleanup(block_hashes)
-            self.node.block_queuing_service.mark_blocks_seen_by_blockchain_node(block_hashes)
+
+        latest_block_number = 0
+        latest_block_difficulty = 0
+
+        block_queuing_service = self.node.block_queuing_service_manager.get_block_queuing_service(self.connection)
+        if block_queuing_service is not None:
+            for block_header in block_headers:
+                block_queuing_service.mark_block_seen_by_blockchain_node(
+                    block_header.hash_object(), None, block_header.number
+                )
+
+                if block_header.number > latest_block_number:
+                    latest_block_number = block_header.number
+                    latest_block_difficulty = block_header.difficulty
+
+        self.node.block_processing_service.set_last_confirmed_block_parameters(
+            latest_block_number, latest_block_difficulty
+        )
 
     def msg_block_bodies(self, msg: BlockBodiesEthProtocolMessage):
         if not self.node.should_process_block_hash():
@@ -248,10 +288,10 @@ class EthNodeConnectionProtocol(EthBaseConnectionProtocol):
                          ", ".join([convert.bytes_to_hex(block_hash.binary) for block_hash in requested_hashes]))
 
             for block_hash, block_body_bytes in zip(requested_hashes, bodies_bytes):
-                if block_hash in self._pending_new_blocks_parts.contents:
+                if block_hash in self.pending_new_block_parts.contents:
                     logger.debug("Received block body for pending new block {}",
                                  convert.bytes_to_hex(block_hash.binary))
-                    self._pending_new_blocks_parts.contents[block_hash].block_body_bytes = block_body_bytes
+                    self.pending_new_block_parts.contents[block_hash].block_body_bytes = block_body_bytes
                     self._check_pending_new_block(block_hash)
                 elif self.node.block_cleanup_service.is_marked_for_cleanup(block_hash):
                     transactions_hashes = \
@@ -271,14 +311,14 @@ class EthNodeConnectionProtocol(EthBaseConnectionProtocol):
 
     def msg_get_receipts(self, msg: GetReceiptsEthProtocolMessage) -> None:
         self.node.log_requested_remote_blocks(msg.get_block_hashes())
-        self.msg_proxy_request(msg)
+        self.msg_proxy_request(msg, self.connection)
 
     def _stop_waiting_checkpoint_headers_request(self):
         self._waiting_checkpoint_headers_request = False
 
     def _check_pending_new_block(self, block_hash):
-        if block_hash in self._pending_new_blocks_parts.contents:
-            pending_new_block = self._pending_new_blocks_parts.contents[block_hash]
+        if block_hash in self.pending_new_block_parts.contents:
+            pending_new_block = self.pending_new_block_parts.contents[block_hash]
 
             if pending_new_block.block_header_bytes is not None and pending_new_block.block_body_bytes is not None:
                 self._ready_new_blocks.append(block_hash)
@@ -286,7 +326,19 @@ class EthNodeConnectionProtocol(EthBaseConnectionProtocol):
     def _process_ready_new_blocks(self):
         while self._ready_new_blocks:
             ready_block_hash = self._ready_new_blocks.pop()
-            pending_new_block = self._pending_new_blocks_parts.contents[ready_block_hash]
+            pending_new_block = self.pending_new_block_parts.contents[ready_block_hash]
+            self.pending_new_block_parts.remove_item(ready_block_hash)
+
+            if ready_block_hash in self.node.blocks_seen.contents:
+                self.node.on_block_seen_by_blockchain_node(
+                    ready_block_hash, self.connection, block_number=pending_new_block.block_number
+                )
+                self.connection.log_info(
+                    "Discarding already seen block {} received in block bodies msg from local blockchain node.",
+                    ready_block_hash
+                )
+                return
+
             last_known_total_difficulty = self.node.try_calculate_total_difficulty(
                 ready_block_hash,
                 pending_new_block
@@ -295,7 +347,6 @@ class EthNodeConnectionProtocol(EthBaseConnectionProtocol):
                 pending_new_block,
                 last_known_total_difficulty
             )
-            self._pending_new_blocks_parts.remove_item(ready_block_hash)
 
             if self.is_valid_block_timestamp(new_block_msg):
                 block_stats.add_block_event_by_block_hash(
@@ -309,8 +360,16 @@ class EthNodeConnectionProtocol(EthBaseConnectionProtocol):
                     ),
                     block_height=new_block_msg.block_number(),
                 )
-                self.node.block_queuing_service.mark_block_seen_by_blockchain_node(
-                    ready_block_hash, new_block_msg
+                gateway_bdn_performance_stats_service.log_block_from_blockchain_node(self.connection.endpoint)
+
+                canceled_recovery = self.node.on_block_seen_by_blockchain_node(
+                    ready_block_hash, self.connection, new_block_msg, pending_new_block.block_number
+                )
+                if canceled_recovery:
+                    return
+
+                self.node.block_queuing_service_manager.push(
+                    ready_block_hash, new_block_msg, node_received_from=self.connection
                 )
                 self.node.block_processing_service.queue_block_for_processing(
                     new_block_msg, self.connection
@@ -348,13 +407,25 @@ class EthNodeConnectionProtocol(EthBaseConnectionProtocol):
         )
 
     def publish_transaction(
-        self, tx_hash: Sha256Hash, tx_contents: memoryview
+        self, tx_hash: Sha256Hash, tx_contents: memoryview, local_region: bool = True
     ) -> None:
+        transaction_feed_stats_service.log_new_transaction(tx_hash)
         self.node.feed_manager.publish_to_feed(
-            EthNewTransactionFeed.NAME,
-            EthRawTransaction(tx_hash, tx_contents, FeedSource.BLOCKCHAIN_SOCKET)
+            FeedKey(EthNewTransactionFeed.NAME),
+            EthRawTransaction(
+                tx_hash,
+                tx_contents,
+                FeedSource.BLOCKCHAIN_SOCKET,
+                local_region=local_region
+            )
         )
+        transaction_feed_stats_service.log_pending_transaction_from_local(tx_hash)
         self.node.feed_manager.publish_to_feed(
-            EthPendingTransactionFeed.NAME,
-            EthRawTransaction(tx_hash, tx_contents, FeedSource.BLOCKCHAIN_SOCKET)
+            FeedKey(EthPendingTransactionFeed.NAME),
+            EthRawTransaction(
+                tx_hash,
+                tx_contents,
+                FeedSource.BLOCKCHAIN_SOCKET,
+                local_region=local_region
+            )
         )
