@@ -2,6 +2,7 @@ import socket
 import time
 from typing import TYPE_CHECKING
 
+from bxcommon.messages.abstract_message import AbstractMessage
 from bxcommon.network.abstract_socket_connection_protocol import AbstractSocketConnectionProtocol
 from bxgateway.connections.abstract_gateway_blockchain_connection import AbstractGatewayBlockchainConnection
 from bxgateway.messages.eth.discovery.eth_discovery_message_factory import eth_discovery_message_factory
@@ -33,13 +34,6 @@ class EthNodeDiscoveryConnection(AbstractGatewayBlockchainConnection["EthGateway
         }
 
         self.can_send_pings = True
-        self.ping_message = PingEthDiscoveryMessage(None,
-                                                    self.node.get_private_key(),
-                                                    eth_common_constants.P2P_PROTOCOL_VERSION,
-                                                    (self.external_ip, self.external_port, self.external_port),
-                                                    (
-                                                    socket.gethostbyname(self.peer_ip), self.peer_port, self.peer_port),
-                                                    int(time.time()) + eth_common_constants.PING_MSG_TTL_SEC)
         self.pong_message = None
 
         self._pong_received = False
@@ -49,6 +43,16 @@ class EthNodeDiscoveryConnection(AbstractGatewayBlockchainConnection["EthGateway
 
         self.hello_messages = [EthDiscoveryMessageType.PING, EthDiscoveryMessageType.PONG]
 
+    def ping_message(self) -> AbstractMessage:
+        return PingEthDiscoveryMessage(
+            None,
+            self.node.get_private_key(),
+            eth_common_constants.P2P_PROTOCOL_VERSION,
+            (self.external_ip, self.external_port, self.external_port),
+            (socket.gethostbyname(self.peer_ip), self.peer_port, self.peer_port),
+            int(time.time()) + eth_common_constants.PING_MSG_TTL_SEC
+        )
+
     def msg_ping(self, msg):
         pass
 
