@@ -1,6 +1,5 @@
 from collections import defaultdict
 
-from bxcommon import constants
 from bxcommon.messages.bloxroute.bloxroute_message_type import BloxrouteMessageType
 from bxgateway.messages.gateway.gateway_message_type import GatewayMessageType
 
@@ -17,7 +16,8 @@ BLOCK_RECOVERY_RECOVERY_INTERVAL_S = [0.1, 0.5, 1, 2, 5]
 # BLOCK_RECOVERY_MAX_RETRY_ATTEMPTS = len(BLOCK_RECOVERY_RECOVERY_INTERVAL_S)
 BLOCK_RECOVERY_MAX_RETRY_ATTEMPTS = 1  # for now, since longer retries aren't really worth it
 BLOCK_RECOVERY_MAX_QUEUE_TIME = 15  # slightly more than sum(BLOCK_RECOVERY_RECOVERY_INTERVAL_S)
-
+CHECK_MEMORY_THRESHOLD_INTERVAL_S = 60 * 60
+CHECK_MEMORY_THRESHOLD_LIMIT = 4 * 1024 * 1024 * 1024
 
 # enum for setting Gateway neutrality assertion policy for releasing encryption keys
 class NeutralityPolicy:
@@ -39,9 +39,10 @@ NEUTRALITY_EXPECTED_RECEIPT_PERCENT = 50
 # previous block in chain. This value can be set to 0 if a blockchain node implementation is capable of
 # immediately taking block messages without validating previous block.
 
-MAX_INTERVAL_BETWEEN_BLOCKS_S = 2
+MAX_INTERVAL_BETWEEN_BLOCKS_S = 0.6
 NODE_READINESS_FOR_BLOCKS_CHECK_INTERVAL_S = 5
 MAX_BLOCK_CACHE_TIME_S = 20 * 60
+MAX_BLOCK_BACKLOG_TO_PUBLISH = 10
 
 GATEWAY_TRANSACTION_STATS_INTERVAL_S = 1 * 60
 GATEWAY_TRANSACTION_STATS_LOOKBACK = 1
@@ -67,8 +68,11 @@ CONFIG_OVERRIDE_NAME = "config.override.json"
 
 BLOCK_HANDLING_TIME_EXPIRATION_TIME_S = 300
 
-INITIAL_LIVELINESS_CHECK_S = 30
+INITIAL_LIVELINESS_CHECK_S = 100
 DEFAULT_STAY_ALIVE_DURATION_S = 30 * 60
+CHECK_BLOCKCHAIN_CONNECTION_FIRMLY_ESTABLISHED = 5 * 60
+
+ACTIVE_BLOCKCHAIN_PEERS_LIVELINESS_CHECK_S = 30 * 60
 
 DEFAULT_BLOCKCHAIN_MESSAGE_TTL_S = 10
 DEFAULT_REMOTE_BLOCKCHAIN_MESSAGE_TTL_S = 10
@@ -82,6 +86,8 @@ BLOCK_CLEANUP_REQUEST_EXPECTED_ADDITIONAL_TRACKED_BLOCKS = 1
 REMOTE_BLOCKCHAIN_MAX_CONNECT_RETRIES = 10
 REMOTE_BLOCKCHAIN_SDN_CONTACT_RETRY_SECONDS = 30
 
+MULTIPLE_BLOCKCHAIN_NODE_MAX_RETRY_S = 300
+
 BLOCK_CONFIRMATION_EXPIRE_TIME_S = 60 * 60
 
 LOGGING_LIMIT_ITEM_COUNT = 10
@@ -89,6 +95,7 @@ LOGGING_LIMIT_ITEM_COUNT = 10
 RELAY_CONNECTION_REEVALUATION_INTERVAL_S = 4 * 60 * 60
 
 BLOCK_QUEUE_LENGTH_LIMIT = 128
+MSG_PROXY_REQUESTER_QUEUE_LIMIT = 15
 
 LOCALHOST = "127.0.0.1"
 
@@ -100,11 +107,10 @@ BLOCKCHAIN_TX_PROCESSING_TIME_WARNING_THRESHOLD_S = 0.05
 WS_DEFAULT_PORT = 28333
 WS_DEFAULT_HOST = LOCALHOST
 RPC_SUBSCRIBER_MAX_QUEUE_SIZE = 1000
-WS_MIN_RECONNECT_TIMEOUT_S = 1
-WS_RECONNECT_TIMEOUTS = [1, 2, 3, 5, 8, 13]
-WS_MAX_CONNECTION_TIMEOUT_S = 2
-
-WS_PROVIDER_MAX_QUEUE_SIZE = 1000
 
 ETH_GAS_RUNNING_AVERAGE_SIZE = 10000
 ADDITIONAL_BLOCKCHAIN_RECONNECT_TIMEOUT_S = 3
+CHECK_RELAY_CONNECTIONS_DELAY_S = 5
+ETH_MIN_GAS_INTERVAL_S = 5 * 60
+
+ETH_PROTOCOL_VERSION_63 = 63
