@@ -1,17 +1,16 @@
 from typing import Tuple, Optional, Union, List
 
-from bxutils import logging
-
+from bxcommon.models.transaction_flag import TransactionFlag
 from bxcommon.messages.bloxroute.tx_message import TxMessage
-from bxcommon.models.quota_type_model import QuotaType
 from bxcommon.utils.blockchain_utils.bdn_tx_to_bx_tx import bdn_tx_to_bx_tx
-from bxgateway.utils.eth.eth_utils import parse_transaction_bytes
+from bxcommon.utils.blockchain_utils.eth import rlp_utils, eth_common_utils
 from bxcommon.utils.object_hash import Sha256Hash
 from bxgateway.abstract_message_converter import AbstractMessageConverter, BlockDecompressionResult
 from bxgateway.messages.eth.internal_eth_block_info import InternalEthBlockInfo
 from bxgateway.messages.eth.protocol.transactions_eth_protocol_message import TransactionsEthProtocolMessage
 from bxgateway.utils.block_info import BlockInfo
-from bxcommon.utils.blockchain_utils.eth import rlp_utils, eth_common_utils
+from bxgateway.utils.eth.eth_utils import parse_transaction_bytes
+from bxutils import logging
 
 logger = logging.get_logger(__name__)
 
@@ -48,7 +47,7 @@ class EthAbstractMessageConverter(AbstractMessageConverter):
         self,
         tx_msg,
         network_num,
-        quota_type: Optional[QuotaType] = None,
+        transaction_flag: Optional[TransactionFlag] = None,
         min_tx_network_fee: int = 0
     ) -> List[Tuple[TxMessage, Sha256Hash, Union[bytearray, memoryview]]]:
         """
@@ -58,7 +57,7 @@ class EthAbstractMessageConverter(AbstractMessageConverter):
 
         :param tx_msg: Ethereum transaction message
         :param network_num: blockchain network number
-        :param quota_type: the quota type to assign to the BDN transaction.
+        :param transaction_flag: the quota type to assign to the BDN transaction.
         :param min_tx_network_fee: minimum transaction fee. excludes transactions with gas price below this value
         :return: array of tuples (transaction message, transaction hash, transaction bytes)
         """
@@ -79,7 +78,7 @@ class EthAbstractMessageConverter(AbstractMessageConverter):
         while True:
             gas_price = eth_common_utils.raw_tx_gas_price(txs_bytes, tx_start_index)
             bx_tx, tx_item_length, tx_item_start = eth_common_utils.raw_tx_to_bx_tx(
-                txs_bytes, tx_start_index, network_num, quota_type
+                txs_bytes, tx_start_index, network_num, transaction_flag
             )
 
             if gas_price >= min_tx_network_fee:
@@ -96,9 +95,9 @@ class EthAbstractMessageConverter(AbstractMessageConverter):
         self,
         raw_tx: Union[bytes, bytearray, memoryview],
         network_num: int,
-        quota_type: Optional[QuotaType] = None
+        transaction_flag: Optional[TransactionFlag] = None
     ) -> TxMessage:
-        return bdn_tx_to_bx_tx(raw_tx, network_num, quota_type)
+        return bdn_tx_to_bx_tx(raw_tx, network_num, transaction_flag)
 
     def bx_tx_to_tx(self, bx_tx_msg):
         """
