@@ -1,6 +1,5 @@
-import rlp
-
-from bxgateway.messages.eth.serializers.block_header import BlockHeader
+import blxr_rlp as rlp
+from bxcommon.messages.eth.serializers.block_header import BlockHeader
 from bxcommon.messages.eth.serializers.transaction import Transaction
 
 
